package andante.agent;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;
import java.io.*;
import java.net.*;

import andante.*;
import andante.orb.*;
import andante.orb.agent.*;
import andante.orb.stage.*;
import java.util.*;

import org.omg.PortableServer.*;
import org.omg.CosNaming.*;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextPackage.*;

public abstract class MusicalAglet extends Aglet {

  protected static int Count = 0;

  protected String Name = null;

  protected boolean back = false;
  protected URL origin = null;
  protected MusicalAglet This;
  protected MusicalAgent ThisServant;

  protected Properties Props;
  protected boolean Playing = false;
  protected boolean WasPlaying = false;

  protected Object Arg;

  protected transient ORB Orb;
  protected transient NamingContextExt LocalNC;
  protected transient NamingContextExt AgentsNC;
  protected transient NameComponent NCName[];
  protected transient Metronome M;
  protected transient boolean Ready = false;

  protected MusicalAgentProxy Proxy = null;

  /***************************************************************************/

  public void init(Stage stage) {
    try {
      AgentsNC = NamingContextExtHelper.narrow(
        LocalNC.resolve(LocalNC.to_name("agents")));
    }
    catch (Exception e) {
      System.out.println("Unhandled exception! Ignoring...");
      e.printStackTrace();
    }
  }

  public void setProperty(String p, String v) {
    Props.setProperty(p, v);
  }
  String getProperty(String p) {
    return Props.getProperty(p);
  }

  String[] listPropertyNames () {
    Set keys = Props.keySet();
    String [] res = new String[keys.size()];
    keys.toArray(res);
    return res;
  }

  Property[] listProperties() {
    Props.list(System.out);
    Set keys = Props.keySet();
    Property[] res = new Property[keys.size()];
    Iterator it = keys.iterator();
    int i = 0;
    while (it.hasNext()) {
      String pn = (String)it.next();
      res[i++] = new Property(pn, Props.getProperty(pn));
    }
    return res;
  }

  public String id() { return getAgletID().toString(); }
  public String name() { return Name; }
  public boolean ready() { return Ready; }

  public void onCreation (Object o) {
    System.out.println("onCreation: " + this.getClass().getName());
    System.out.println("onCreation: arg: " + o);

    Arg = o;
    This = this;

    Count++;

    try {
      if (o != null && Class.forName("java.util.Map").isInstance(o)) {
        System.out.println("onCreation: received args table");
        Map args = (Map)o;
        if (args.containsKey("Proxy")) {
          Proxy = (MusicalAgentProxy)args.get("Proxy");
        }
      }
    }
    catch (Exception e) { 
      System.out.println("What?!?");
      e.printStackTrace();
    }

    if (Name == null) {
      Name = This.getClass().getName() + " #" + Count;
    }

    Props = new Properties();

    addMobilityListener (
      new MobilityAdapter () {
        public void onArrival (MobilityEvent me) {
	  System.out.println("onArrival");

	  if (WasPlaying) Playing = true;

	  Stage stage = 
	    (Stage)getAgletContext().getProperty("Stage");
	  Orb = (ORB)getAgletContext().getProperty("Orb");
	  LocalNC = (NamingContextExt)getAgletContext().getProperty("NamingContext");

	  try {
	    POA Poa = org.omg.PortableServer.POAHelper.narrow
	      ( Orb.resolve_initial_references("RootPOA"));
	    Poa.the_POAManager().activate();
	    MusicalAgentImpl ma = new MusicalAgentImpl(This);
	    org.omg.CORBA.Object obj = Poa.servant_to_reference(ma);
	    ThisServant = MusicalAgentHelper.narrow(obj);

            if (Proxy != null)
              Proxy.setMusicalAgent(This.id(), ThisServant);

            // this must go before 'init':
            AgentsNC = NamingContextExtHelper.narrow(
              LocalNC.resolve(LocalNC.to_name("agents")));

            init(stage);

	    NCName = LocalNC.to_name(ma.name());
	    AgentsNC.rebind(NCName, obj);


	  }
	  catch (Exception e) {
            System.out.println("Unhandled exception! Ignoring...");
	    e.printStackTrace();
	  }

          synchronized (This) {
            System.out.println("Agent ok...");            
            Ready = true;
            This.notifyAll();
          }
          
	  if (WasPlaying) play();
        }
      }
      ); //addMobilityListener

    origin = getAgletContext().getHostingURL();
    synchronized (this) {
      Ready = true;
      notifyAll();
    }
  }

  public void processSequence (andante.orb.Note[] seq) {
  }

  public void processNote (andante.orb.Note n) {
  }

  public void processMessage (andante.orb.Message m) {
  }

  public void stop() {
  }
  public void play() {
  }

  boolean playing () {
    return Playing;
  }

  public final void dispatch(String address) {
    try {
      if (Playing) {
	WasPlaying = true;
	stop();
      }
      else WasPlaying = false;

      if (LocalNC != null)
        LocalNC.unbind(NCName);
      System.out.println("Dispatching " + address);
      dispatch(new URL(address));
    } catch (Exception e) { e.printStackTrace(); }
  }


  public void pulse() {
  }

  public static void main(String args[]) {
    try {
      //proxy.dispose();
      //AgletsServer.addStartupAglet("MusicalAglet");
      AgletProxy proxy;
      AgletsServer.main(args);
//      proxy = AgletsServer.getContext().createAglet
//	(null, "andante.MAMidiFile", new FileInputStream("../bak/rach3-1.mid"));
//      AgletsServer.getContext().createAglet
//	(null, "andante.MAMidiFile", new FileInputStream("../bak/actr-botp.mid"));
//      AgletsServer.getContext().createAglet
//	(null, "andante.MAMidiFile", new FileInputStream("../bak/smwwd2.mid"));
//      AgletsServer.getContext().createAglet
///	(null, "andante.MADodge", null);
//      proxy = AgletsServer.getContext().createAglet
//	(null, "andante.MANoise", null);
//      AgletsServer.getContext().createAglet
//	(null, "andante.MANoise", null);
//      AgletsServer.getContext().createAglet
//	(null, "andante.MANoise", null);
//      new URL("atp://localdomain:444");

//      proxy.dispatch(new URL("atp://localhost:4444"));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

}
