package andante.device;

import org.omg.PortableServer.*;
import andante.orb.device.*;
import andante.orb.*;

import javax.sound.midi.*;
import java.util.*;
import java.io.*;

public class JavaMidiDevice extends AudioDevicePOA {

  String Id;

  Synthesizer Synth;
  MidiChannel[] Canais;

  Timer T;

  MidiChannel C;

  short PlayNoteTab[][];

  public JavaMidiDevice() {
    init();
  }

  protected void init() {
    try {
      Id = "Default MIDI Device";
      Synth = MidiSystem.getSynthesizer();
      Synth.open();
      Canais = Synth.getChannels();
      
      PlayNoteTab = new short[Canais.length][128];
      
      T = new Timer();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public String id () { return Id; }

  public void noteOn(int channel, int pitch, int vel) {
    Canais[channel].noteOn(pitch, vel);
  }
  public void noteOff(int channel, int pitch) {
    Canais[channel].noteOff(pitch);
  }

  public void sendMessage (int channel, andante.orb.Message m) {
    if (m.type.value() == MessageType.NOTE_ON.value()) {
      Canais[channel].noteOn(m.data[0], m.data[1]);
    }
    else if (m.type.value() == MessageType.NOTE_OFF.value()) {
      Canais[channel].noteOff(m.data[0], m.data[1]);
    }
    else if (m.type.value() == MessageType.NOTE_PLAY.value()) {
      Canais[channel].noteOn(m.data[0], m.data[1]);
      PlayNoteTab[channel][m.data[0]] += m.data[1];
      T.schedule(new TTNoteOff_(channel, m.data[0] , m.data[1]),
		     m.data[2]);
    }
  }

  class TTNoteOff_ extends TimerTask {    
    int C;
    int P;
    int I;
    public TTNoteOff_(int channel, int pitch, int intensity) {
      C = channel;
      P = pitch;
      I = intensity;
    }    
    public void run() {
      PlayNoteTab[C][P] -= I;
      Canais[C].noteOn(P, min(PlayNoteTab[C][P], (short)128));
      
    }
  }

  public static final short min(short a, short b) {
    return (a < b)? a:b;
  }

  public void processSequence (andante.orb.Event[] seq) {
    try {
      Sequencer s = MidiSystem.getSequencer();

      Sequence sequence = new Sequence(Sequence.PPQ, 10);
      Track track = sequence.createTrack();

      SequenciaDeEventsToTrack(seq, track);

      MidiSystem.write(sequence, (MidiSystem.getMidiFileTypes(sequence))[0],
		       new File("out.mid"));
      s.open();
      s.setSequence(sequence);
      s.start();

    } catch (Exception ex) { ex.printStackTrace(); }

  }

  /**
   * Transforma um vetor de andante.orb.Event em um Track.
   */
  public static void
  SequenciaDeEventsToTrack (andante.orb.Event[] seq, Track track)
  {
    try {
      for (int i = 0; i < seq.length; i++) {
	ShortMessage message = new ShortMessage();    
	if (seq[i].message.type.value() == MessageType.NOTE_ON.value()) {
	  message.setMessage(ShortMessage.NOTE_ON,
			     seq[i].message.data[0], seq[i].message.data[1]);
	  MidiEvent event = new MidiEvent(message, seq[i].tempo);
	  track.add(event);
	}
	else if (seq[i].message.type.value() == MessageType.NOTE_OFF.value()) {
	  message.setMessage(ShortMessage.NOTE_OFF,
			     seq[i].message.data[0], seq[i].message.data[1]);
	  MidiEvent event = new MidiEvent(message, seq[i].tempo);
	  track.add(event);
	}
	else if (seq[i].message.type.value() == MessageType.NOTE_PLAY.value()) {
	  message.setMessage(ShortMessage.NOTE_ON,
			     seq[i].message.data[0], 
			     seq[i].message.data[1]);
	  MidiEvent event = new MidiEvent(message, seq[i].tempo);
	  track.add(event);
	  message = new ShortMessage();
	  message.setMessage(ShortMessage.NOTE_OFF,
			     seq[i].message.data[0],
			     seq[i].message.data[1]);
	  event = new MidiEvent(message, seq[i].tempo + seq[i].message.data[2] + 10);
	  track.add(event);
	}
      }
    } catch (Exception ex) { ex.printStackTrace(); }
  }


  public void playMidiFile (byte[] arq) {
    System.out.println("DA: tocaArquivoMIDI " + arq.length);
/*    try {
      Sequencer seq = MidiSystem.getSequencer();
      InputStream is = new BufferedInputStream(byteArrayToInputStream(arq));

	      MidiDevice.Info	seqInfo = getMidiDeviceInfo("Tritonus ALSA Sequencer", true);
	      if (seqInfo == null)
		{
		  System.out.println("Cannot find device ");
		  System.exit(1);
		}
	      seq = (Sequencer) MidiSystem.getMidiDevice(seqInfo);

      seq.open();
      seq.getTransmitter().setReceiver(new MyReceiver());
      seq.setSequence(is);
      seq.start();

    } catch (Exception e) {
      e.printStackTrace();
    }
*/
  }

  /***************************************************************************/

  public static void main(String args[]) {
    org.omg.CORBA.ORB Orb = org.omg.CORBA.ORB.init(args, null); 
    JavaMidiDevice da = new JavaMidiDevice();

    try {
      POA Poa = org.omg.PortableServer.POAHelper.narrow
	( Orb.resolve_initial_references("RootPOA"));
      Poa.the_POAManager().activate();
      org.omg.CORBA.Object obj = Poa.servant_to_reference(da);

      String arqIor = "DA.ref";
      System.out.println("Imprimindo a IOR do Dispositivo no arquivo " +
			 arqIor + "...");
      PrintWriter ps = new PrintWriter
	( new FileOutputStream( new File( arqIor )));
      ps.println( Orb.object_to_string( obj ) );
      ps.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    Orb.run();
  }

  /***************************************************************************/

  public void changeInstrument(int channel, short instr) {
    Canais[channel].programChange(instr);
  }



  /***************************************************************************/
  /*
   *	This method tries to return a MidiDevice.Info whose name
   *	matches the passed name. If no matching MidiDevice.Info is
   *	found, null is returned.
   *	If forOutput is true, then only output devices are searched,
   *	otherwise only input devices.
   */
  private static MidiDevice.Info getMidiDeviceInfo(String strDeviceName, boolean forOutput) {
    MidiDevice.Info[]	aInfos = MidiSystem.getMidiDeviceInfo();
    for (int i = 0; i < aInfos.length; i++) {
      if (aInfos[i].getName().equals(strDeviceName)) {
	try {
	  MidiDevice device = MidiSystem.getMidiDevice(aInfos[i]);
	  boolean	bAllowsInput = (device.getMaxTransmitters() != 0);
	  boolean	bAllowsOutput = (device.getMaxReceivers() != 0);
	  if ((bAllowsOutput && forOutput) || (bAllowsInput && !forOutput)) {
	    return aInfos[i];
	  }
	} catch (MidiUnavailableException mue) {}
      }
    }
    return null;
  }

  public static InputStream byteArrayToInputStream(byte [] v) {
    PipedInputStream pis = null;
    try {
      pis = new PipedInputStream();
      PipedOutputStream pos = new PipedOutputStream(pis);

      (new Escritor(v, pos)).start();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return pis;
  }

  private static class Escritor extends Thread {
    byte [] V;
    OutputStream Out;

    public Escritor(byte v [], OutputStream out) {
      V = v;
      Out = out;
    }
    public void run() {
      try {
        Out.write(V, 0, V.length);
        Out.close();
      } catch (Exception e) { e.printStackTrace(); }
    }
  }
}
