package andante.stage;

import java.io.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import andante.orb.agent.*;
import andante.orb.stage.*;
import andante.orb.device.*;
import andante.orb.*;

public class MetronomeImpl extends MetronomePOA {

  Channel C = null;
  int CNum = 9;

  int TimeSigAmount;
  int TimeSigType;
  int Tempo; // beats per min

  int BeatInMillis;
  int Count;

  List Agents;

  boolean PlayPulse;
  Note StrongPulse;
  Note WeakPulse;
  
  Stage S;
  DoIt Ticker;

  public MetronomeImpl(Stage s, int tsa, int tst, int tempo) {
    S = s;
    TimeSigAmount = tsa;
    TimeSigType = tst;
    setTempo(tempo);
    Count = 0;
    Agents = new Vector();
    PlayPulse = false;
    StrongPulse = new Note();
    setNote(StrongPulse, 60, 120, 100);
    WeakPulse = new Note();
    setNote(WeakPulse, 60, 70, 100);
    Ticker = new DoIt();
    Ticker.start();
  }

  public void register (andante.orb.agent.MusicalAgent ma) {
    Agents.add(ma);
  } 
  public void unregister (andante.orb.agent.MusicalAgent ma) {
    Agents.remove(ma);
  } 

  public void synchronize() {
    DoIt old = Ticker;
    Ticker = new DoIt();
    Ticker.start();
    old.end();
  }

  public int getPulseMillis() { return BeatInMillis; }

  class DoIt extends Thread {
    boolean Stop = false;

    public void end() {
      Stop = true;
    }
    public void run() {

      Iterator it;


      long tt1 = System.currentTimeMillis();
      long tt2;

      while (!Stop) {
	tt2 =  System.currentTimeMillis();
//	System.out.println(tt2 - tt1);
	tt1 = tt2;

	if (PlayPulse) {
	  if (Count % TimeSigAmount == 0) {
	    C.noteOn(StrongPulse.pitch,StrongPulse.intensity);
	  }
	  else {
	    C.noteOn(WeakPulse.pitch,WeakPulse.intensity);
	  }
	}
      
	it = Agents.iterator();
	while (it.hasNext()) {
	  MusicalAgent ma = (MusicalAgent)it.next();
	  ma.pulse(); 
	}

	try { Thread.sleep(BeatInMillis); }
	catch (InterruptedException ie) { System.err.println("Interrupted!"); }

	if (PlayPulse) {
	  if (Count % TimeSigAmount == 0) {
	    C.noteOff(StrongPulse.pitch,StrongPulse.intensity);
	  }
	  else {
	    C.noteOff(WeakPulse.pitch,WeakPulse.intensity);
	  }
	}

	Count++;
      }
    }

  }

  public void setTempo(int tempo) {
    Tempo = tempo;
    BeatInMillis = (60000 / Tempo); // 'Tempo' is in bpm
  }
  public int getTempo() { return Tempo; }

  public void setTimeSignature(int tsa, int tst) {
    TimeSigAmount = tsa;
    TimeSigType = tst;
  }
  public void setTSAmount(int tsa) { TimeSigAmount = tsa; }
  public void setTSType(int tst) { TimeSigType = tst; }
  public int getTSAmount() { return TimeSigAmount; }
  public int getTSType() { return TimeSigType; }

  public void playPulses(boolean play) {
    if (C == null) {
      C = S.channel(CNum);
    }
    PlayPulse = play;
  }
  public boolean isPlayingPulses() { return PlayPulse; }

  public void setStrongPulse(int p, int i, int d) {
    setNote(StrongPulse, p, i ,d);
  }
  public void setWeakPulse(int p, int i, int d) {
    setNote(WeakPulse, p, i ,d);
  }
  public void setStrongPulse(short p, short i, int d) {
    setNote(StrongPulse, p, i ,d);
  }
  public void setWeakPulse(short p, short i, int d) {
    setNote(WeakPulse, p, i ,d);
  }

  public void setNote(Note n, int pitch, int intensity, int duration) {
    n.pitch = (short)(pitch % 128);
    n.intensity = (short)(intensity % 128);
    n.duration = (short)(duration % 128);
  }
}
