package andante.test;

import java.applet.Applet;
import java.awt.Button;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;

/**
 * This applet is an example of use of the SocketAdapter class.
 * 
 * 'Port' specifies the port on which an instance of SocketAdapter is
 * listening. The applet will open a socket to localhost on this port.
 * 'Message' is the message sent to the SocketAdapter. This message is
 * converted to a CORBA message and then sent to the agent associated with
 * the SocketAdapter. 
 */
public class SAApplet extends Applet {

  TextArea taOutput;
  TextField tfPort;
  TextField tfMsg;
  Button bSend;

  Socket S;

  PrintStream psMsg;
  BufferedReader brMsg;
  boolean brMsgOk = false;

  String Newline;

  /***************************************************************************/

  public void init() {

    Newline = System.getProperty("line.separator");

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints gc = new GridBagConstraints();
    setLayout(gridbag);

    // Port panel

    Panel pPort = new Panel();
    pPort.setLayout(new FlowLayout(FlowLayout.LEFT));
    tfPort = new TextField("5000", 6);
    tfPort.setEditable(true);
    Button bSetPort = new Button("Set");

    pPort.add(new Label("Port:"));
    pPort.add(tfPort);
    pPort.add(bSetPort);

    bSetPort.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setPort(tfPort.getText());
      }
    });

    gc.fill = GridBagConstraints.HORIZONTAL;
    gc.weighty = 0.0;
    gc.gridx = 0;
    gc.gridy = 0;

    gridbag.setConstraints(pPort, gc);
    add(pPort);

    // Message panel

    Panel pMsg = new Panel();
    pMsg.setLayout(new FlowLayout(FlowLayout.LEFT));
    tfMsg = new TextField("setProperty Scale Chromatic", 20);
    tfMsg.setEditable(true);
    bSend = new Button("Send");
    bSend.setEnabled(false);

    bSend.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        sendMessage(tfMsg.getText());
      }
    });

    pMsg.add(new Label("Message:"));
    pMsg.add(tfMsg);
    pMsg.add(bSend);

    gc.fill = GridBagConstraints.HORIZONTAL;
    gc.weighty = 0.0;
    gc.gridx = 0;
    gc.gridy = 1;

    gridbag.setConstraints(pMsg, gc);
    add(pMsg);

    // Debug output panel

    Panel pOut = new Panel();
    pOut.setLayout(new GridLayout(1, 0));
    taOutput = new TextArea();
    taOutput.setEditable(true);
    pOut.add(taOutput);

    gc.fill = GridBagConstraints.BOTH;
    gc.weighty = 1.0;
    gc.weightx = 1.0;
    gc.gridx = 0;
    gc.gridy = 2;

    gridbag.setConstraints(pOut, gc);
    add(pOut, gc);

    validate(); //this shouldn't be necessary

    // The following thread receives messages back from the SocketAdapter
    // (for debugging purposes, not really necessary):
     (new Thread() {
      public void run() {
        while (true) {
          try {
            String msg = brMsg.readLine();
            println("** Returned: " + msg);
            if (msg == null) {
              closeSocket();
              println("Connection closed.");
              waitForBr();
            }
          } catch (Exception e) {
            waitForBr();
          }
        }
      }
      private void waitForBr() {
        synchronized (taOutput) {
          while (!brMsgOk) {
            try {
              taOutput.wait();
            } catch (Exception ee) {
            }
          }
        }
      }
    }).start();
  }

  public void start() {
  }

  public void stop() {
  }

  public void destroy() {
    try {
      closeSocket();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /***************************************************************************/

  private void setPort(String port) {
    try {
      int p = Integer.parseInt(port);
      closeSocket();
      S = new Socket(getCodeBase().getHost(), p);
      println("Conection opened: " + S);
      psMsg = new PrintStream(S.getOutputStream());
      brMsg = new BufferedReader(new InputStreamReader(S.getInputStream()));
      synchronized (taOutput) {
        brMsgOk = true;
        taOutput.notify();
      }
      bSend.setEnabled(true);
    } catch (Exception e) {
      println("Error: " + e);
      //e.printStackTrace();
    }
  }

  private void closeSocket() throws Exception {
    synchronized (taOutput) {
      brMsgOk = false;
    }
    if (S != null) {
      psMsg.close();
      brMsg.close();
      S.close();
    }
    bSend.setEnabled(false);
  }
  
  private void sendMessage(String msg) {
    psMsg.println(msg);
    psMsg.flush();
  }

  private void println(String msg) {
    print(msg + Newline);
  }

  private void print(String msg) {
    taOutput.append(msg);
  }
}
