package andante.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Properties;
import java.util.StringTokenizer;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.Binding;
import org.omg.CosNaming.BindingIteratorHolder;
import org.omg.CosNaming.BindingListHolder;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;

import andante.orb.agent.MusicalAgent;
import andante.orb.agent.MusicalAgentHelper;

/**
 * This class is part of an example that shows a possible way to communicate 
 * with Andante agents.
 * 
 * An instance of <code>SocketAdapter</code> receives messages from a TCP/IP
 * socket and translates them to CORBA messages sent to an Andante agent.
 */
public class SocketAdapter {

  MusicalAgent MA;
  ServerSocket SS;

  public SocketAdapter(MusicalAgent ma, int port) {
    MA = ma;
    try {
      SS = new ServerSocket(port);
    } catch (IOException ioe) {
      System.err.println("Error opening socket: " + ioe);
      System.exit(1);
    }

    (new Thread() {
      public void run() {
        while (true) {
          Socket conn = null;
          try {
            conn = SS.accept();
            System.out.println("Connection established: " + conn);
            System.out.println();

            Adapter t = new Adapter(conn);
            t.start();
          } catch (IOException e) {
            e.printStackTrace();
          }
        }
      }
    }).start();

  }

  public int getLocalPort() {
    return SS.getLocalPort();
  }

  class Adapter extends Thread {

    protected Socket S;
    protected BufferedReader In;
    protected PrintStream Out;

    public Adapter(Socket socket) {
      S = socket;
      try {
        In = new BufferedReader(new InputStreamReader(S.getInputStream()));
        Out = new PrintStream(S.getOutputStream(), true);
      } catch (IOException e) {
        e.printStackTrace();
        System.exit(1);
      }
    }

    public void run() {

      try {
        while (true) {
          String msg = In.readLine();
          System.out.println(msg);
          if (msg == null)
            break;
          StringTokenizer st = new StringTokenizer(msg);

          if (!st.hasMoreTokens())
            continue;

          String method = st.nextToken();
          if (method.equals("setProperty")) {
            // assume msg is ok...
            MA.setProperty(st.nextToken(), st.nextToken());
            Out.println("Message sent.");
            Out.flush();
          } else if (method.equals("play")) {
            MA.play();
            Out.println("Message sent.");
            Out.flush();
          } else if (method.equals("stop")) {
            MA.stop();
            Out.println("Message sent.");
            Out.flush();
          } else if (method.equals("quit")) {
            break;
          } else {
            Out.println("Message NOT sent.");
            Out.flush();
          }
        }

        S.close();
        System.out.println("Connection closed: " + S);
        System.out.println();

      } catch (Exception e) {
        e.printStackTrace();
      }
    }

  }

  /***************************************************************************/
  
  public static void main(String args[]) {

    try {
      Properties props = new Properties();
      props.put("org.omg.CORBA.ORBInitialHost", "localhost");
      props.put("org.omg.CORBA.ORBInitialPort", "1050");
      ORB orb = ORB.init(args, props);

      NamingContextExt ncR =
        NamingContextExtHelper.narrow(
          orb.resolve_initial_references("NameService"));
      NamingContextExt nc =
        NamingContextExtHelper.narrow(ncR.resolve_str("andante/stages"));

      BindingListHolder bl = new BindingListHolder();
      BindingIteratorHolder blIt = new BindingIteratorHolder();
      nc.list(1, bl, blIt);

      Binding bindings[] = bl.value;
      if (bindings.length == 0)
        throw new Error("No stages found.");

      System.out.println(
        bindings[0].binding_name[bindings[0].binding_name.length
          - 1]
          + " found.");

      NamingContextExt nc2 =
        NamingContextExtHelper.narrow(nc.resolve(bindings[0].binding_name));
      nc2 =
        NamingContextExtHelper.narrow(nc2.resolve_str("agents/NoiseWeaver"));

      nc2.list(1, bl, blIt);
      bindings = bl.value;

      if (bindings.length == 0)
        throw new Error("No agent found.");

      System.out.println(
        bindings[0].binding_name[bindings[0].binding_name.length
          - 1]
          + " found.");

      org.omg.CORBA.Object obj = nc2.resolve(bindings[0].binding_name);
      MusicalAgent ma = MusicalAgentHelper.narrow(obj);

      SocketAdapter sa = new SocketAdapter(ma, Integer.parseInt(args[0]));
      System.out.println("Waiting for messages...");

    } catch (Exception ex) {
      ex.printStackTrace();
    }

  }
}
